/////////////////////Perspective Shadowmapping by Redphoenix///////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////

//Tweakables
static const float fDark = 0.1;
static const float fBright = 1.0;
static const float fDepthOffset = 0.997;

// Application fed data:
const float4x4 matWorldViewProj;	// World*view*projection matrix.
const float4x4 matWorld;	// World matrix.
const float4x4 matView;	// World matrix.
const float4x4 matViewProj;	// World matrix.
const float4x4 matWorldProj;	// World matrix.
const float4x4 matProj;	// World matrix.

const float4x4 matWorldView;	// World matrix.
const float4 vecSunDir;	// Sun direction vector.
float3 vecSunPos;
float4x4 matMtl; 	// Texture projection matrix
//float4x4 matEffect; 	// Texture projection matrix

texture mtlSkin1;
texture entSkin1;
float4 vecSkill1;
float4 vecSkill5;
sampler DepthSampler = sampler_state { Texture = <mtlSkin1>; };
sampler TexSampler = sampler_state { Texture = <entSkin1>; Mipfilter = Linear; };

// Shadow mapping vertex shader
void ShadowVS (in float4 inPos: POSITION,
		in float4 inTex: TEXCOORD0,
		in float3 inNormal: NORMAL,
		out float4 outPos: POSITION,
		out float4 outTex: TEXCOORD0,
		out float3 outNormal: TEXCOORD1,
		out float4 outDepth: TEXCOORD2)
{
// Transform the vertex from object space to clip space:
	outPos = mul(mul(inPos,matWorld),matViewProj);
// Transform the normal from object space to world space:
	outNormal = normalize(mul(inNormal,matWorld));
// Pass the texture coordinate to the pixel shader:
	outTex = inTex;
	outDepth = mul(mul(inPos,matWorld),matMtl);
// Output the projective texture coordinates
 /*
   float4 gShadowViewerPos = vecSkill5.xyzw;
   float2 texDiff = outDepth.xy - gShadowViewerPos.xy; 
   float t = length( texDiff)*0.00004f;
	float nt = gShadowViewerPos.w * log( gShadowViewerPos.z * t + 1) / t;
   outDepth.xy = gShadowViewerPos.xy + texDiff * nt;
   */
}

#define USE_PCF

#ifdef USE_PCF
static const float4 fTaps_PCF[9] = {
	{-1.0,-1.0, 0.0, 0.0},
	{-1.0, 0.0, 0.0, 0.0},
	{-1.0, 1.0, 0.0, 0.0},
	{ 0.0,-1.0, 0.0, 0.0},
	{ 0.0, 0.0, 0.0, 0.0},
	{ 0.0, 1.0, 0.0, 0.0},
	{ 1.0,-1.0, 0.0, 0.0},
	{ 1.0, 0.0, 0.0, 0.0},
	{ 1.0, 1.0, 0.0, 0.0}};
#endif

// distance comparison function
float fDist(float4 DepthCoord,float fDepth)
{
	return tex2Dproj(DepthSampler,DepthCoord).r < (fDepth*vecSkill1.z)? vecSkill1.x : fBright;
}

// Shadow mapping pixel shader
float4 ShadowPS (in float4 inTex: TEXCOORD0,
		in float3 inNormal: TEXCOORD1,
		in float4 inDepth: TEXCOORD2) : COLOR0
{
// Calculate the diffuse term:
	float fDiffuse = saturate(dot(-vecSunDir, normalize(inNormal)));

// Calculate the shadow term
#ifdef USE_PCF
	float fShadow = 0.0;
	for (int i=0; i < 9; i++)
	{
		float4 fTap = inDepth + fTaps_PCF[i] * vecSkill1.y;
		fShadow += fDist(fTap,inDepth.z);
	}
	fShadow /= 9;
#else	
	float fShadow = fDist(inDepth,inDepth.z);
#endif		
	
	
	return  float4(fShadow*fDiffuse,fShadow*fDiffuse,fShadow*fDiffuse,1.0); /* fDiffuse*/;
	//return  float4(fShadow/*fDiffuse*/,fShadow/*fDiffuse*/,fShadow/*fDiffuse*/,1.0); /* fDiffuse*/;
	//return float4(tex2Dproj(DepthSampler,inDepth).r/2,0.0,0.0,1.0);
//return float4(tex2D(DepthSampler,inTex).rgb*2000,1.0);
//return float4(inDepth.x/20000,0.0,0.0,1.0);
//return float4(inDepth.z*100,0.0,0.0,1.0);
//return 1.0;
}

technique techShadow{	pass p0	{   /*CullMode = CCW;*/	AlphaBlendEnable = False;ZWriteEnable = True;
	VertexShader = compile vs_2_0 ShadowVS();		PixelShader  = compile ps_2_0 ShadowPS();	}}

