//////////////////////////////////////////////////
///Shader for rendering Variance Shadow Mapping///
//////////////////////////////////////////////////
//////////////////////////////////////////////////
////14.01.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

// Constants given by the engine
float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matLight1;

float4 vecProjViewPos;
float4 vecValues;

float xLightPower = 1.0;
float Ambient = 0.2;


// Textures
Texture mtlSkin1;
sampler Depth1Sampler = sampler_state
{
	texture = <mtlSkin1>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter = LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

texture entSkin1;
sampler ColorMapSampler = sampler_state		// Color map sampler.
{
	Texture = <entSkin1>;
	MipFilter = LINEAR;
};

// Structs
struct SSceneVertexToPixel
{
	float4 Position					: POSITION;
	float4 ShadowMapSamplingPos1	: TEXCOORD0;
	float2 TexCoords					: TEXCOORD1;
};

// Vertex Shader
SSceneVertexToPixel ShadowedSceneVertexShader( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0, float3 inNormal : NORMAL)
{
	SSceneVertexToPixel Output = (SSceneVertexToPixel)0;
	
	Output.Position = mul(inPos, matWorldViewProj);
	Output.ShadowMapSamplingPos1 = mul(inPos,mul(matWorld,matLight1));
	
///////////////////////////////////////////////////////////////////////////
	// Verzerrungszentrum wieder in gSchattenBetrachterPos gegeben, in .zw liegen wieder die Konstanten
	// Abstand zum Verzerrungszentrum
	float2 diff = Output.ShadowMapSamplingPos1.xy - vecProjViewPos.xy;
	float t = length(diff)*0.002;
	
	// Verzerrung anwenden
	float nt =  vecProjViewPos.w * log(vecProjViewPos.z * t + 1) / t;
	
	// in neue Vertexposition umrechnen
	Output.ShadowMapSamplingPos1.xy = vecProjViewPos.xy + diff * nt;
///////////////////////////////////////////////////////////////////////////

	Output.TexCoords.xy = inTexCoords;
	
	return Output;
}

// Pixel Shader
float VSM_FILTER(float2 moments, float fragDepth)
{   moments.x =  moments.x*0.9998601;
	float Ex_2 = moments.x * moments.x;
	float variance = moments.y - Ex_2;
	float mD = moments.x - fragDepth;
	float mD_2 = mD * mD;
	float p = variance / (variance + mD_2);

	return max(p, fragDepth <= moments.x );
}

float4 ShadowedScenePixelShader(SSceneVertexToPixel PSIn) : COLOR
{
	float3 ProjectedTexCoords1;
	PSIn.ShadowMapSamplingPos1 = PSIn.ShadowMapSamplingPos1 / PSIn.ShadowMapSamplingPos1.w;
	ProjectedTexCoords1.xy = PSIn.ShadowMapSamplingPos1.xy * float2(0.5, -0.5) + 0.5;
	ProjectedTexCoords1.z = PSIn.ShadowMapSamplingPos1.z;

	float4 Shadow = 0.5+(VSM_FILTER(tex2D(Depth1Sampler, ProjectedTexCoords1.xy).rg,ProjectedTexCoords1.z-0.0000001));//(tex2D(Depth1Sampler, ProjectedTexCoords1.xy).r > ProjectedTexCoords1.z);
	Shadow.a = 1.0; //tex2D(ColorMapSampler,PSIn.TexCoords.xy).a;
	
	return Shadow;
}


// Technique
technique ShadowedScene
{
	pass Pass0
	{/*
		ZWriteEnable = True;
		AlphaBlendEnable = False;
		AlphaTestEnable = True;
		AlphaRef = 97;
		AlphaFunc = Greater;
		*/
		VertexShader = compile vs_2_0 ShadowedSceneVertexShader();
		PixelShader = compile ps_2_0 ShadowedScenePixelShader();
	}
}