// cre_hmp.cpp : Definiert den Einstiegspunkt fr die Konsolenanwendung.
//

///////////////////////////////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <math.h.>

// engine specific header file
#define DLL_USE	// always define before including adll.h
#include "adll.h"	
#include "hmp5.h"


BOOL APIENTRY DllMain( 
  HANDLE hModule,
  DWORD ul_reason_for_call,
  LPVOID lpReserved)
{
  engine_bind();
  return TRUE;
}


////////////////////////////////////////////////////////////////////////////

HMP7W DllHmpW;

//starts writing of an HMP
DLLFUNC var WriteHMP (CHAR* filename)
{
	void*	handle;
 	handle = DllHmpW.Open(filename);
	if(handle == NULL) { return(_VAR(0)); }
	return(_VAR(1));
}

//Writes the Header of an HMP
DLLFUNC var WriteHmpHeader(var header_info[9],VECTOR* min,VECTOR* max)
{
	HMP5_HEADER* header; 

	header = &DllHmpW.Head();
	DllHmpW.Head().fnumverts_x = _FLOAT(header_info[0]);
	DllHmpW.Head().ftrisize_x = _FLOAT(header_info[1]);
	DllHmpW.Head().ftrisize_y = _FLOAT(header_info[2]);

	float v_origin[3];
	float v_scale[3];

	v_origin[0] = _FLOAT(min->x); v_origin[1] = _FLOAT(min->y); 
       v_origin[2] = _FLOAT(min->z);

	v_scale[0] = _FLOAT(max->x) - v_origin[0]; 
       v_scale[1] = _FLOAT(max->y) - v_origin[1];
       v_scale[2] = _FLOAT(max->z) - v_origin[2];

	v_scale[0] /= 65535.0f;
	v_scale[1] /= 65535.0f;
	v_scale[2] /= 65535.0f; 

	DllHmpW.Head().scale[0] = v_scale[0];
	DllHmpW.Head().scale[1] = v_scale[1];
	DllHmpW.Head().scale[2] = v_scale[2];
	DllHmpW.Head().scale_origin[0] = v_origin[0];
	DllHmpW.Head().scale_origin[1] = v_origin[1];
	DllHmpW.Head().scale_origin[2] = v_origin[2]; 

	return(_VAR(1));
}

//finish hmp writing 
DLLFUNC var WriteHmpClose()
{
	bool a = DllHmpW.Close();
	return(_VAR((int)a));
}

//starts a frame
DLLFUNC var WriteHmpFrame(STRING* framename)
{
	bool a = DllHmpW.Frame(framename->chars);
	return(_VAR((int)a));
}

//writes one vertex
DLLFUNC var WriteHmpVertex(var height, VECTOR* normal)
{
	unsigned short packed_z = 0;
	if(DllHmpW.Head().scale[2] == 0)
	{
		packed_z = 0;
	}
	else
	{
		packed_z = (short) ((_FLOAT(height) - DllHmpW.Head().scale_origin[2]) / DllHmpW.Head().scale[2]);
	}
//converted to hmp7 format containing normalinformation
	bool a = DllHmpW.Vertex(packed_z,_FLOAT(normal->x),_FLOAT(normal->y),_FLOAT(normal->z));
	return(_VAR(packed_z));
}

#define	SKINTYP_8BIT_PAL256			0x00
#define	SKINTYP_16BIT_565				0x02
#define	SKINTYP_16BIT_4444			0x03
#define	SKINTYP_24BIT_888				0x04
#define SKINTYP_32BIT_8888			0x05

#define	SKINTYP_MIPFLAG				0x08
#define	SKINTYP_MATERIAL				0x10
#define	SKINTYP_MATERIAL_ASCDEF		0x20
#define	SKINTYP_RGBFLAG				0x80 

#define	SKINTYP_16BIT_565M			(SKINTYP_16BIT_565 | SKINTYP_MIPFLAG)
#define	SKINTYP_16BIT_4444M			(SKINTYP_16BIT_4444 | SKINTYP_MIPFLAG)
#define	SKINTYP_24BIT_888M			(SKINTYP_24BIT_888 | SKINTYP_MIPFLAG)
#define	SKINTYP_32BIT_8888M			(SKINTYP_32BIT_8888 | SKINTYP_MIPFLAG)


//generates a mipmap
bool Create_MipMap(BYTE* mip,BYTE* bmp,int width, int height,int bpp, int scalefactor)
{
	int a = 0; int b = height-1; int c = 0; int d = 0;
	while(b > 0)
	{
		while (d < width*bpp) {
		    c = 0;
		    while(c < bpp)
		    {
			    mip[a+c] = bmp[b*width+d+c];
			    c += 1;
		    }
		    a += 1;
			d+=scalefactor*bpp;
		}
		d %= width*bpp;
	 	b -= 1;
	} 

	return(true);
}

//writes a skin
DLLFUNC var WriteHmpSkin(STRING* str,BMAP* bmp)
{
	int bpp = 0;
	int type = 0;
	bool mips = 0;
	int format = _INT(bmap_lock(bmp,0));
	switch ( format )
	{
	case 565: type = SKINTYP_16BIT_565*((bmp->flags & BMPF_MIPMAPS) ==0) 
       + SKINTYP_16BIT_565M*((bmp->flags & BMPF_MIPMAPS) !=0);  break; 
	case 4444: type = SKINTYP_16BIT_4444*((bmp->flags & BMPF_MIPMAPS) ==0) 
       + SKINTYP_16BIT_4444M*((bmp->flags & BMPF_MIPMAPS)!=0); break;
	case 888: type = SKINTYP_24BIT_888*((bmp->flags & BMPF_MIPMAPS) ==0) 
       + SKINTYP_24BIT_888M*((bmp->flags & BMPF_MIPMAPS)!=0); break;
	case 8888: type = SKINTYP_32BIT_8888*((bmp->flags & BMPF_MIPMAPS) ==0) 
       + SKINTYP_32BIT_8888M*((bmp->flags & BMPF_MIPMAPS)!=0); break;
	default: 
		return _VAR(-1);
	}; 

	if(bmp->bytespp == 3 && type == SKINTYP_32BIT_8888M)
	{
		type = SKINTYP_24BIT_888M;
	}
	if(bmp->bytespp == 3 && type == SKINTYP_32BIT_8888)
	{
		type = SKINTYP_24BIT_888;
	}  

	//mips = (bool) ((int)bmp->flags & BMPF_MIPMAPS);
	mips = (bool) (true); //always create mipmaps
	if (type == SKINTYP_32BIT_8888) { type = SKINTYP_32BIT_8888M; }
	if (type == SKINTYP_24BIT_888) { type = SKINTYP_24BIT_888M; }
	bpp = bmp->bytespp;
	if(!bpp || !type) { return(_VAR(-1)); } 

	int size = (_INT(bmap_width(bmp))) * (_INT(bmap_height(bmp))) * bpp;
	BYTE* nmip = new BYTE[size];
	int g = 0;
	for(int d=_INT(bmap_height(bmp))-1;d>=0;d-=1) {
		//load the bmap upside down because .tgas origin is specifed wrong by GS
		for (int e=0;e<_INT(bmap_width(bmp));e++) {
			for(int f=0;f<bpp;f++) {
		      nmip[g] = bmp->pixels[d*_INT(bmap_width(bmp))*bpp+e*bpp+f]; 
			  g+=1;
			}
		}
	}
	bool erfolg;
	int mtrl_typ = type;
	DllHmpW.Skin ( mtrl_typ, str->chars);
	DllHmpW.SkinSize ( _INT(bmap_width(bmp)), _INT(bmap_height(bmp)), bpp, mips );
	if(mips == true)
	{
	 	int size = (_INT(bmap_width(bmp))/2) * (_INT(bmap_height(bmp))/2) * bpp;
		BYTE* mip2 = new BYTE[size];
		Create_MipMap(mip2,bmp->pixels,_INT(bmap_width(bmp)),_INT(bmap_height(bmp)),bpp,4); 

		size = (_INT(bmap_width(bmp))/4) * (_INT(bmap_height(bmp))/4) * bpp;
		BYTE* mip3 = new BYTE[size];
		Create_MipMap(mip3,bmp->pixels,_INT(bmap_width(bmp)),_INT(bmap_height(bmp)),bpp,16); 

		size = (_INT(bmap_width(bmp))/8) * (_INT(bmap_height(bmp))/8) * bpp;
		BYTE* mip4 = new BYTE[size];
		Create_MipMap(mip4,bmp->pixels,_INT(bmap_width(bmp)),_INT(bmap_height(bmp)),bpp,64); 

		erfolg = DllHmpW.SkinBits ( nmip, mip2, mip3, mip4); 

		delete mip2;
		delete mip3;
		delete mip4;
	}
	else
	{
		erfolg = DllHmpW.SkinBits ( nmip, NULL,NULL,NULL);
	} 

	DllHmpW.SkinDone();
	bmap_unlock(bmp);  

	return(_VAR((int)erfolg));
}

//writes an empty skin (1 skin is needed)
DLLFUNC var WriteSkinEmpty(var width2,var height2,STRING* str)
{
	int width = _INT(width2);
	int height = _INT(height2);
	int type = SKINTYP_8BIT_PAL256;
	int bpp = 1;

	DllHmpW.Skin ( type, str->chars);
	DllHmpW.SkinSize ( width, height, bpp, false );
	int size = width*height*bpp;
	BYTE* skin = new BYTE[size];
	memset(skin,0,size);  

	DllHmpW.SkinBits (skin, NULL, NULL, NULL);
	DllHmpW.SkinDone(); 

	return(_VAR(1));
}
 
//Close an open frame
DLLFUNC var WriteHmpFrameEnd()
{
	bool a = DllHmpW.FrameDone();
	return(_VAR((int)a));
}
