////////////////////////////////Water shader for Worldsforge///////////////////////////////////

#define MIRROR_FRESNEL
float4x4 matWorld;
float4x4 matWorldView;
float4x4 matWorldViewProj;
float4 vecFog;
float4 vecViewPos;
float4 DoTransform(float4 Pos)
{
	return mul(Pos,matWorldViewProj);
}
#ifdef DXFOG
float DoFog(float4 Pos)
{
	float3 P = mul(Pos,matWorldView); // convert vector to view space to get it's depth (.z)
   	return saturate((vecFog.y-P.z) * vecFog.z); // apply the linear fog formula
}
#else // distance based fog
float DoFog(float4 Pos)
{
	float3 P = mul(Pos,matWorld);
	return 1 - (distance(P,vecViewPos)-vecFog.x) * vecFog.z;
}
#endif
float4 DoPos(float4 Pos)
{
	return (mul(Pos,matWorld));
}
float3 DoPos(float3 Pos)
{
	return (mul(Pos,matWorld));
}
float3 DoNormal(float3 inNormal)
{
	return normalize(mul(inNormal,matWorld));
}	
	float4 vecTime;
	float4 vecSkill41; // Wind_X, Wind_Y, Ripple, Scale
	float4 vecColor;	
	float4 vecSkill1; // ss
	float4 vecSkill5; // ss
	float4 vecSkill9; // ss

	texture mtlSkin1;
	texture mtlSkin2;
	texture mtlSkin3;
	texture mtlSkin4;

	sampler sBumpTex = sampler_state
	{
		Texture = <mtlSkin1>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Wrap;
		Addressv = Wrap;
	};

	sampler sMirrorTex = sampler_state
	{
		Texture = <mtlSkin2>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Clamp;
		Addressv = Clamp;
	};
	sampler sBump2Tex = sampler_state
	{
		Texture = <mtlSkin3>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Clamp;
		Addressv = Clamp;
	};
	sampler sShoreTex = sampler_state
	{
		Texture = <mtlSkin4>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Wrap;
		Addressv = Wrap;
	};
	struct out_water_mirror // Output to the pixelshader fragment
	{
		float4 Pos	: POSITION;
		float4 Color: COLOR0;
		float  Fog	: FOG;
		float2 Tex0	: TEXCOORD0;
		float4 Tex1	: TEXCOORD1;
		float4 realTex	: TEXCOORD2;

	};


float wave_amplitude = 4;//0.01
float wave_frequency = 60;//60
float fAlpha;
float RippleWaveFunc( float2 point, float2 center, float radius, float frequency, float amplitude, float power )
{

float dist = length( point - center );
float delta = max(0,radius - dist);

#ifdef ASHLI
// ashli cos gives wrong output with input outside -PI,PI range
float cosv = cos( fmod(delta * frequency, 2.0*PI) - PI );
#else
float cosv = cos( delta*frequency );
#endif

float magnitude = cosv * ( amplitude/ pow(1+delta,power) );
return magnitude;
}

// computes displacement of a point in UV space
float Ripple( float2 texcoords, float2 ripple_center, float frequency, float amplitude, float power)
{
float wavefront_dist = amplitude + (vecTime.w*0.002); //*vecTime_scale
return RippleWaveFunc( texcoords, ripple_center, wavefront_dist, frequency, amplitude, power)/**20*/;
}

float DisplacementFunc( float2 texcoords )
{
float frequency = vecSkill1.x*4;
float amplitude = vecSkill1.y;
float2 ripple_center = normalize(vecSkill1.zw);//RIPPLE POSITION(CENTER)
//float2 ripple_center = float2(/*vecSkill1.y*/0.8,0.8);
float r1 = Ripple(texcoords, ripple_center ,frequency,amplitude,vecSkill5.x/10);
return r1 ; 
}
/*
float3 CalculateNormal( float3 N, float3 tanU, float2 texcoords )
{
// V tangent = Normal crossed with U tangent
float3 tanV = normalize(cross(tanU, N));

const float DELTA_U = 0.00001;
const float DELTA_V = 0.00001;

float mag = DisplacementFunc(texcoords);
float magU = DisplacementFunc( float2(texcoords[0] + DELTA_U , texcoords[1]) );
float magV = DisplacementFunc( float2(texcoords[0], texcoords[1] + DELTA_V) );

float dfdu = (magU-mag) / DELTA_U;
float dfdv = (magV-mag) / DELTA_V;

float nu = -dfdu;
float nv = -dfdv;

float3 newN = normalize(nu*tanU + nv*tanV + N);

if( dot(newN,N) < 0 )
newN = -newN;

return newN;
}
*/
float4 DisplaceSurface( float4 P, float3 N, float magnitude )
{
// displace based on the location in the X/Z plane
float4 Pp = P + (magnitude*float4(N,0));
return Pp;
}

	out_water_mirror vs_water_mirror
	(
		in float4 inPos		: POSITION,
		in float3 inNormal	: NORMAL,
		in float4 inTex0	: TEXCOORD0
	)
	{
		out_water_mirror Out;
		
		float magnitude = DisplacementFunc( float2(inTex0[0],inTex0[1]) );
      float3 N = DoNormal(inNormal); // normal world orientation
      float3 Nn = normalize(inNormal); 
      float4 nP = DisplaceSurface(inPos, Nn, magnitude);
      //Out.Normal = normalize(CalculateNormal(Nn, normalize(intanU), float2(inTex0[0],inTex0[1])));

		
		Out.Pos = DoTransform(nP); // transform to screen coordinates
		Out.Fog = DoFog(nP);
// bump and reflection coordinates
		Out.Tex0 = (inTex0 + vecSkill41.xy * vecTime.w) * vecSkill41.w;
		Out.Tex1 = Out.Pos.xyzw;
		Out.realTex = inTex0;

// color and transparency
		Out.Color = vecColor;
#ifdef MIRROR_FRESNEL
		float4 P = DoPos(nP); // vector world position
		float4 vecToView = normalize(vecViewPos-P); // direction towards camera
		Out.Color.a = 0.5 + fAlpha * (1.0 - dot(vecToView,N))+distance(inPos,vecViewPos)/1400;
#endif		
		return Out;
	}
	

	float4 ps_mirror_straight(out_water_mirror In): COLOR
	{ 
		return tex2D(sMirrorTex,In.Tex1) * In.Color;
	}
	
	float4 ps_water_mirror(out_water_mirror In): COLOR
	{
		float3 temp=In.Tex1.xyz/In.Tex1.w;
		float3 Bump = 2*tex2D(sBumpTex,In.Tex0).xyz-1;
	   float3 Bump2 = 2*tex2D(sBumpTex,In.Tex0*0.1).xyz-1;
	   float3 Bump3 = 2*tex2D(sBumpTex,In.Tex0*10).xyz-1;
	   float2 ripCoord = saturate((0.5+(In.realTex-vecSkill5.zw)*(vecSkill5.y-(vecTime.w-vecSkill9.y)/(2))/2));
	   float3 ripBump = float3(0.0,0.0,0.0);
	   float ripDist = distance(ripCoord.xy,float2(0.5,0.5));
      if ((ripCoord.x > 0.0 && ripCoord.x < 1.0) && (ripCoord.y > 0.0 && ripCoord.y < 1.0) && (ripDist < 0.4)) { ripBump = 2*tex2D(sBump2Tex,ripCoord).xyz-1; }
      ripBump = lerp(ripBump*pow(1-saturate(3*ripDist),3),ripBump*pow(saturate(2*ripDist),3),saturate(abs(vecTime.w-vecSkill9.y)/(160)));
      ripBump = lerp(ripBump,0.0,saturate(abs(vecTime.w-vecSkill9.y)/(160)));
      ripBump = lerp(0.0,ripBump,saturate(abs(vecTime.w-vecSkill9.y)/(80)));
      //ripBump *= pow(abs(vecTime.w-vecSkill5.y)*vecSkill5.z,saturate(length(ripCoord)));
		float3 Coord=0.5*(1.0+temp)+Bump.xyz*vecSkill41.z+Bump2.xyz*vecSkill41.z*0.5+Bump3.xyz*vecSkill41.z*2/*+ripBump.xyz*/;
	   float4 shore_color;
	   shore_color.r = tex2D(sShoreTex,In.realTex).b;
	   if (shore_color.r > 0) {
	   shore_color.g = tex2D(sShoreTex,(In.realTex + vecSkill41.xy*sin(vecTime.w*0.03)*50) * vecSkill41.w).r;	
	   shore_color.b = tex2D(sShoreTex,(In.realTex + vecSkill41.xy*cos(vecTime.w*0.03)*50) * vecSkill41.w).g;
	   shore_color.a = (shore_color.g+shore_color.b-1);	
	   }
		float4 final = tex2D(sMirrorTex,Coord)*In.Color;
		float4 Shore_Color = float4 (1,1,1,shore_color.a);
		return (lerp(final,Shore_Color,shore_color.r));
		//return(final);
	}


//////////////////////////////////////////////////////////////////
	technique water_mirror
	{
		pass one
		{
			AlphaBlendEnable = True;

			sampler[0] = (sBumpTex);
			sampler[1] = (sMirrorTex);
			sampler[2] = (sBump2Tex);
			sampler[3] = (sShoreTex);

			VertexShader = compile vs_1_1 vs_water_mirror();
			PixelShader = compile ps_2_0 ps_water_mirror();
			
		}
	}

	//technique fallback { pass one { } }



